#!/usr/bin/python
# test_blockhosts.py

'''
Unit tests for blockhosts.py

Run from current directory, should have blockhosts.py and
blockhosts.cfg files and the test_data/ directory available.
Command:
   python blockhosts_tests.py

'''

TEST_BLOCKFILE='test_data/test_hosts.allow'
TEST_LOG_FILE='test_data/test_all.log'

# TEST_CONFIG_FILE='test_data/test_v203.cfg' # failing matches, version 2.0.3
TEST_CONFIG_FILE= 'blockhosts.cfg'

SCRIPT_ID='test_blockhosts'
VERSION='0.0.5'
VERSION_DATE='June 2007'
AUTHOR='Avinash Chopde'
AUTHOR_EMAIL='avinash@acm.org'
URL='http://www.aczoom.com/cms/blockhosts/'
LICENSE='http://creativecommons.org/licenses/publicdomain/'
DESCRIPTION='test blockhosts.py functionality'
LONG_DESCRIPTION=DESCRIPTION


import unittest
import StringIO
import syslog
import sys
import re

import blockhosts
from blockhosts import Log

class TestBlockHosts(unittest.TestCase):

    BH_OBJ = None
    CONFIG = None

    def setUp(self):

        #Log.SetPrintLevel(Log.MESSAGE_LEVEL_DEBUG)
        Log.SetPrintLevel(Log.MESSAGE_LEVEL_INFO)
        #Log.SetPrintLevel(0) # disable all non-error messages

        if self.BH_OBJ:
            return

        args = ['--quiet', '--configfile=' + TEST_CONFIG_FILE]

        config = blockhosts.Config(args, VERSION, LONG_DESCRIPTION)
        config.add_section(blockhosts.CommonConfig())
        config.add_section(blockhosts.BlockHostsConfig())
        config.add_section(blockhosts.MailConfig())
        config.add_section(blockhosts.IPBlockConfig())

        config.parse_args()

        Log.Info('%s %s started: %s' % (SCRIPT_ID, VERSION, blockhosts.Config.START_TIME_STR))

        bh_obj = blockhosts.BlockHosts(config['blockfile'],
                                       config['blockline'])

        TestBlockHosts.CONFIG = config
        TestBlockHosts.BH_OBJ = bh_obj

    def testLogsMatch(self):

        # All these IP addresses should be matched, and no others
        table = (
            '10.100.1.1', '10.100.1.2', '10.100.1.3', '10.100.1.4',
            '10.100.1.5', '10.100.1.6', '10.100.1.7',
            '10.101.1.1', '10.101.1.2', '10.101.1.3', '10.101.1.4',
            '10.102.1.1', '10.102.1.3', '10.102.1.4', '10.102.1.5',
            '10.102.1.6', '10.102.1.7', '10.102.1.8',
            '10.103.1.1',
            '10.104.1.1', '10.104.1.2',
            '10.105.1.1',
            '10.106.1.1',
            '10.107.1.1',
            )

        Log.Info(' Test scanning log')

        failed = [] # maintain list of failure messages

        sl = blockhosts.SystemLog(TEST_LOG_FILE)

        offset = blockhosts.SystemLogOffset(0,'') # offset 0, read from stat
        sl.open(offset)

        all_regexs = {}
        # compile all regular expression patterns - ignore
        # ENABLE_PATTERNS, and enable all 
        for (name, rule) in self.CONFIG['ALL_REGEXS_STR'].iteritems():
            all_regexs[name] = re.compile(rule)

        while 1:
            line = sl.readline()
            if not line: break

            line = line.strip()
            if not line: continue

            host_entry = self.BH_OBJ.match_line(line, all_regexs)
            if host_entry:
                ip = host_entry[0]
                count = host_entry[1].count
                Log.Debug('  ... checking IP', ip, count)
                if ip not in table:
                    failed.append('** Watched IP %s unexpected, in line:\n   %s' % (ip, line))
            
        sl.close()

        (blocked, watched) = self.BH_OBJ.get_hosts_lists()
        watched_ips = watched.keys()
        for ip in table:
            if ip not in watched_ips:
                # check that no required IP address is missing
                failed.append('** Required IP %s not matched in log' % (ip))
            else:
                # check that counts are correct
                d1, d2, d3, d4 = (int(d) for d in ip.split('.'))
                count = watched[ip].count
                if count != d3:
                    failed.append('** Watched IP %s count %d not %d' % (ip, count, d3))

        self.failIf(failed, 'Scanning failed, Errors:\n%s' % ('\n'.join(failed)))

    def testMain(self):
        args = [ '--dry-run', '--logfiles=' + TEST_LOG_FILE,
                 '--quiet',  '--configfile=' + TEST_CONFIG_FILE,
                 '--blockfile=' + TEST_BLOCKFILE,
               ]

        Log.Info(' Test main')
        Log.Debug('  ... main(%s)' % (args))

        output = StringIO.StringIO()
        save_stdout = sys.stdout
        save_stderr = sys.stderr
        sys.stdout = output
        sys.stderr = output
        try:
            blockhosts.main(args)
        finally:
            sys.stdout = save_stdout
            sys.stderr = save_stderr

        value = output.getvalue()
        # print 'got stdout and stderr', output.getvalue()

        self.failUnless(value.upper().find('ERROR:') < 0,
                        'main() output contains ERROR:\n%s' %
                        value)

        self.failUnless(value.upper().find('WARNING:') < 0,
                        'main() output contains Warning:\n%s' %
                        value)

    def testBHrss(self):
        args = [ '--quiet',  '--configfile=' + TEST_CONFIG_FILE,
                 '--blockfile=' + TEST_BLOCKFILE,
               ]

        import bhrss

        Log.Info(' Test bhrss')
        Log.Debug('  ... main(%s)' % (args))

        output = StringIO.StringIO()
        save_stdout = sys.stdout
        save_stderr = sys.stderr
        sys.stdout = output
        sys.stderr = output
        try:
            bhrss.main(args)
        finally:
            sys.stdout = save_stdout
            sys.stderr = save_stderr

        value = output.getvalue()
        # print 'got stdout+stderr: ', output.getvalue()

        self.failUnless(value.upper().find('<CHANNEL>') > 0,
                        'main() output does not contain channel:\n%s' %
                        value)


if __name__ == '__main__':
    syslog.openlog(SCRIPT_ID)
    # don't syslog anything (EMERG is 0, so can't use it, use ALERT)
    syslog.setlogmask(syslog.LOG_ALERT)
    unittest.main()
